import express, { type Express } from 'express';
import cors from 'cors';
import compression from 'compression';
import {Logger} from "tslog";
import helmet from "helmet";
import AuthRouter from "@routes/auth/authRouter";
import CatalogRouter from "@routes/catalog/catalogRouter";
import RentRouter from "@routes/rent/rentRouter";
import * as process from "node:process";


const logger = new Logger({ name: "App" });

const app: Express = express();

// enable cors
app.use(cors());
app.options('*', cors());

// enable xss sanitizer
app.use(
	helmet({
		xXssProtection: true,
	}),
);
app.use(helmet.xXssProtection())

// parse json request body
app.use(express.json());

// parse urlencoded request body
app.use(express.urlencoded({ extended: true }));

// gzip compression
app.use(compression())

try {
	app.use('/auth', AuthRouter)
	app.use('/catalog', CatalogRouter)
	app.use('/rent', RentRouter)
	logger.info('Routers loaded !')
} catch (err) {
	logger.error(err)
	throw null;
}

try {
	app.listen(process.env["APP_PORT"])
	logger.info('Server is running !')
} catch (error) {
	logger.error(`Server failed to start: ${error}`);
  process.exit(1);
}